using System.Collections.Generic;
using Orbiter.Interfaces;

namespace Orbiter.Multiplayer
{
    public class Vessel:GlobalObject
    {
        private string navigationInfo;
        private string dockingInfo;
        private string className;
        private Dictionary<string, string> eventInfo;
        private User user;
        private NTPTime lastInfoPingTime;
        private StateInfoPacket state;
        
        public Vessel(Server server, User user):base(server)
        {
            this.user = user;
            lastInfoPingTime=new NTPTime(0,0); //Start information gathering
            eventInfo=new Dictionary<string, string>();
        }
        
        public void Update(StateInfoPacket pac)
        {
            lock (this)
            {
                state = pac;
                lock(Server.Map.Cluster)
                {
                    Server.Map.Cluster.Set(this,
                                       pac.LogarithmicScale,
                                       user.Time.Update(user, pac),
                                       new Vector3(pac.Position.X, pac.Position.Y, pac.Position.Z));
                }
                Host userHost = user.Host;
                if (lastInfoPingTime != null && Server.NTP.Now - lastInfoPingTime > 5 && userHost != null)
                {
                    userHost.Send("REQST " + GlobalId + "\n\r");
                    lastInfoPingTime = Server.NTP.Now;
                }
            }
        }

        public User User
        {
            get { return user; }
        }

        public string ClassName
        {
            get { return className; }
        }

        public StateInfoPacket State
        {
            get
            {
                return state;
            }
        }

        public override bool EvaluateInfo(params string[] strings)
        {
            lock (this)
            {
                if (strings.Length > 0 && strings[0].Length > 0) switch (strings[0][0])
                {
                    case 'C':
                        if (lastInfoPingTime != null && strings[0].Length > 1 && strings.Length > 1)
                        {
                            Name = strings[0].Substring(1);
                            className = Server.Map.GetVesselClass(strings[1]);
                            lastInfoPingTime = null;
                        }
                        break;
                    case 'N':
                        navigationInfo = strings[0].Substring(1);
                        return true;
                    case 'D':
                        dockingInfo = strings[0].Substring(1);
                        return true;
                    case 'E':
                        var elements = strings[0].Substring(1).Split('"');
                        if (elements.Length<2) return false;
                        if (!eventInfo.ContainsKey(elements[1])) eventInfo.Add(elements[1], strings[0]);
                        else eventInfo[elements[1]] = strings[0];
                        return true;
                }
                return false;
            }
        }

        public override string[] GenerateInfo()
        {
            lock (this)
            {
                var strings = new List<string>(3+eventInfo.Count) {"\"C" + Name + '"' + ClassName};
                if (navigationInfo != null) strings.Add("\"N" + navigationInfo);
                if (dockingInfo != null) strings.Add("\"D" + dockingInfo);
                foreach (var info in eventInfo) strings.Add("\"E" + info.Value);
                return strings.ToArray();
            }
        }
    }
}
