//############################################################################//
#define STRICT
#define ORBITER_MODULE
//############################################################################//
#include <stdarg.h>
#include <orbitersdk.h>
#include <windows.h>
#include "RacerCheckpointMFD.h"
//############################################################################//
//############################################################################//
#define RED    RGB(255,0,0)
#define GREEN  RGB(0,255,0)
#define BLUE   RGB(0,0,255)
#define WHITE  RGB(255,255,255)
#define GRAY   RGB(180,180,180)
#define YELLOW RGB(255,255,224)  
//############################################################################//
//############################################################################//
int mode=0;
std::vector<MFDData*> g_data;
//############################################################################//
//############################################################################//
void TextO(HDC hdc,int wid,int &y,int yi,const char* fmt,...)
{   
 char cbuf[255];   
 va_list arg;
 va_start(arg,fmt);
 int len=vsprintf(cbuf,fmt,arg);
 va_end(arg); 
 TextOut(hdc,wid,y,cbuf,len);
 y+=yi;
}
//############################################################################//
void Prune()
{
 if(g_data.size()==0)return;
 std::vector<MFDData*>::iterator it=g_data.begin();
 for (UINT i=0;i<g_data.size();i++){
  it=g_data.begin()+i;
  if (g_data[i]==0){
   g_data.erase(it);
  }else{
   if (!oapiIsVessel(g_data[i]->hook->getHook())){
    delete g_data[i];
    g_data.erase(it);
   }
  }
 }
}    
DWORD GetExtensionCharCount(char * file)
{
	DWORD len = strlen(file);
	if (len < 2) // can't have an extention 
		return 0; 
	DWORD count = 0;
	for (DWORD i = len; i > 0; i--)
	{
		if (file[i] == '.')
			break;
		else
			count++;
	}
	return (count == len ? 0 : count);
}
//############################################################################//
bool GetDouble(void *id,char *str,void *data)
{
 if (strlen(str)==0)return false;
 *((double*)data)=atof(str);
 return true;
}   
//############################################################################//
bool GetFileName(void *idc,char *str,void *data)
{
 if(strlen(str)==0)return false;
 DWORD extCount = GetExtensionCharCount(str);
 char filename[255];
 if (extCount > 0)
	 sprintf(filename,"checkpoints/%s",str);
 else
	 sprintf(filename,"checkpoints/%s.chkpt",str);
 FILE * fp=fopen(filename,"r");
 if(fp==0)return false;
 fclose(fp);
 ((MFDData*)data)->hook->LoadCheckpoints(filename);
 return true;
}          
//############################################################################//
//############################################################################//
//############################################################################//
DLLCLBK void opcPreStep(double simt,double simdt,double mjd)
{
 Prune();
 for(UINT i=0;i<g_data.size();i++)g_data[i]->TimeStep();
}        
//############################################################################//
DLLCLBK void opcDLLInit(HINSTANCE hDLL)
{
 static char *name="Racer Checkpoint MFD";
 
 MFDMODESPEC spec;
 spec.name   =name;
 spec.key    =OAPI_KEY_R;
 spec.msgproc=RCPMFD::MsgProc;
 
 mode=oapiRegisterMFDMode(spec);
}
//############################################################################//
DLLCLBK void opcDLLExit (HINSTANCE hDLL)
{
 for (UINT i=0;i<g_data.size();i++)if(g_data[i])delete g_data[i];
 g_data.clear();
 oapiUnregisterMFDMode(mode);
}  
//############################################################################//
//############################################################################// 
//##############################     MFD     #################################//
//############################################################################//
//############################################################################//
RCPMFD::RCPMFD(DWORD w,DWORD h,VESSEL *vessel):MFD(w,h,vessel)
{
 bool found=false;
 for (UINT i=0;i<g_data.size();i++){
  if (vessel->GetHandle()==g_data[i]->hook->getHook()){
   found=true;
   this->data=g_data[i];
   break;
  }
 }
 if(!found){
  MFDData * dt=new MFDData(vessel->GetHandle());
  g_data.push_back(dt);
  data=dt;
 }
 
 height=h/20;
 width=w/35;
 wm=w;
 hm=h;
} 
//############################################################################//
void RCPMFD::ReadStatus(FILEHANDLE scn)
{
	char *line,str[255];
	while(oapiReadScenario_nextline(scn,line))
	{
		if(!_strnicmp(line,"END_MFD",7))return;
		if(!_strnicmp(line,"TRACK",5))
		{
			sscanf(line+5,"%s",&str);
			if (strcasecmp(str,"none"))
			{
				FILE *fp=fopen(str,"r");
				if(fp!=0){fclose(fp);((MFDData*)data)->hook->LoadCheckpoints(str);}
			}
		}else if(!_strnicmp(line,"STATUS",6))
		{
			int st=0;
			sscanf(line+6,"%d",&st);
			if((VFSTATUS)st!=IDLE)
			{
				data->hook->toggleStatus();
				data->hook->setMST(oapiGetSimTime());
			}
		}
		else if(!_strnicmp(line,"MET",3))
		{
			float met=0;
			sscanf(line+3,"%f",&met);
			data->hook->setMST(-met);
		}else if(!_strnicmp(line,"CHKP",4))
		{
			int chkp=0;
			sscanf(line+4,"%d",&chkp);
			data->hook->currentCHKPT=chkp;
		}else if(!_strnicmp(line, "TIMER", 5))
		{
			int tm = 0;
			sscanf(line+5, "%d", &tm);
			data->hook->setTimer(tm);
		}
	}
}
//############################################################################//
void RCPMFD::WriteStatus(FILEHANDLE scn) const
{
 // The following MUST be in this order
 oapiWriteScenario_string(scn,"TRACK",data->hook->getFileName());
 oapiWriteScenario_int   (scn,"STATUS",(int)data->hook->getStatus());
 oapiWriteScenario_float (scn,"MET",data->hook->getMET());
 oapiWriteScenario_int   (scn,"CHKP",data->hook->currentCHKPT); 
 oapiWriteScenario_int   (scn,"TIMER", data->hook->getTimer());
}
//############################################################################//
//############################################################################//
RCPMFD::~RCPMFD(){}   
//############################################################################//
int RCPMFD::MsgProc(UINT msg,UINT mfd,WPARAM wparam,LPARAM lparam)
{
 switch(msg){
  case OAPI_MSG_MFD_OPENED:return (int)(new RCPMFD (LOWORD(wparam),HIWORD(wparam),(VESSEL*)lparam));
 }
 return 0;
}   
//############################################################################//
bool RCPMFD::ConsumeKeyBuffered (DWORD key)
{
 // page independant buttons
 switch(key)
 {
 case OAPI_KEY_P:
  {
   data->nextPage();
   InvalidateDisplay();
   InvalidateButtons();
   return true;
  }
 }
 switch(data->page)
 {
 case PAGE_MAIN:
  {
   switch(key)
   {
   case OAPI_KEY_S:
    {
     data->hook->toggleStatus();
     return true;
    }
   case OAPI_KEY_L:
    {
     oapiOpenInputBox("Enter checkpoint filename: (ex. 'habana.chkpt')",GetFileName,0,20,data);
     return true;
    }
   case OAPI_KEY_R:
    {
     data->hook->reset();
     return true;
    }
   case OAPI_KEY_LBRACKET:
    {
     UINT iCount=data->hook->getCheckpointCount();
     if (iCount==0)
      return false;
     if (data->iSelectedCheckPoint == 0)
      data->iSelectedCheckPoint=data->hook->getCheckpointCount()-1;
     else
      data->iSelectedCheckPoint--;
     return true;
    } 
   case OAPI_KEY_RBRACKET:
    {
     UINT iCount=data->hook->getCheckpointCount();
     if (data->iSelectedCheckPoint+1 >= iCount)
      data->iSelectedCheckPoint=0;
     else
      data->iSelectedCheckPoint++;
     return true;
    }
   } break;
  }
 case PAGE_VESSELINFO:
  {
   switch(key)
   {
   case OAPI_KEY_LBRACKET:
    {
     data->prevTarget();
     return true;
    }
   case OAPI_KEY_RBRACKET:
    {
     data->nextTarget();
     return true;
    }
   }
   break;
  }
 case PAGE_RACEMAP:{
  switch(key){  
   case OAPI_KEY_Z:{data->zoom*=2;return true;}
   case OAPI_KEY_X:{data->zoom/=2;return true;}
  }
  break;
 }
 }
 return false;
}
//############################################################################//
bool RCPMFD::ConsumeButton(int bt,int event)
{
 if (!(event & PANEL_MOUSE_LBDOWN)) return false;

 switch(data->page)
 {
 case PAGE_MAIN:
  {
   switch(bt){
    case 0:return ConsumeKeyBuffered(OAPI_KEY_P);
    case 1:return ConsumeKeyBuffered(OAPI_KEY_S);
    case 2:return ConsumeKeyBuffered(OAPI_KEY_L);
    case 3:return ConsumeKeyBuffered(OAPI_KEY_R);
    case 4:return ConsumeKeyBuffered(OAPI_KEY_LBRACKET);
    case 5:return ConsumeKeyBuffered(OAPI_KEY_RBRACKET);
   }
   break;
  }
 case PAGE_VESSELINFO:
  {
   switch(bt){
    case 0:return ConsumeKeyBuffered(OAPI_KEY_P);
    case 1:return ConsumeKeyBuffered(OAPI_KEY_LBRACKET);
    case 2:return ConsumeKeyBuffered(OAPI_KEY_RBRACKET);
   }
   break;
  }
 case PAGE_RACEMAP:
  {
   switch(bt){
    case 0:return ConsumeKeyBuffered(OAPI_KEY_P);
    case 1:return ConsumeKeyBuffered(OAPI_KEY_Z);
    case 2:return ConsumeKeyBuffered(OAPI_KEY_X);
   }
   break;
  }
 }
 return false;
}
//############################################################################//
//############################################################################//
char *RCPMFD::ButtonLabel(int bt)
{
 switch(data->page)
 {
 case PAGE_MAIN:
  {
   char *label[6]={"PG","TOG","LOD","RES","CK-","CK+"};
   return (bt<6 ? label[bt] : 0);
  }
 case PAGE_VESSELINFO:
  {
   char *label[3]={"PG","TG-","TG+"};
   return (bt<3 ? label[bt] : 0);
  }
 case PAGE_RACEMAP:
  {
   char *label[3]={"PG","ZIN","ZOU"};
   return(bt<3 ? label[bt] : 0);
  }
 }
 return 0;
}
//############################################################################//
int RCPMFD::ButtonMenu(const MFDBUTTONMENU **menu) const
{
 switch(data->page){
 case PAGE_MAIN:{
   static const MFDBUTTONMENU mnu[6]={
    {"switch page",0,'P'},
    {"start/pause race",0,'S'},
    {"load checkpoint file",0,'L'},
    {"reset race",0,'R'},
    {"info previous checkpoint",0,'['},
    {"info next checkpoint",0,']'}
   };
   if(menu) *menu=mnu;
   return 6;
  }
 case PAGE_VESSELINFO:{
   static const MFDBUTTONMENU mnu[3]={
    {"switch page",0,'P'},
    {"prev. target",0,'['},
    {"next target",0,']'}
   };
   if(menu) *menu=mnu;
   return 3;
  }
 case PAGE_RACEMAP:{
   static const MFDBUTTONMENU mnu[3]={
    {"switch page",0,'P'},
    {"zoom in",0,'Z'},
    {"zoom out",0,'X'}
   };
   if(menu) *menu=mnu;
   return 3;
  }
 }
 return 0;
}   
//############################################################################//
//############################################################################//
void RCPMFD::DrawPageMain(HDC hDC)
{
 int y=height;

 if (data->hook->isRaceFinished()==TRUE)
 {
  SetTextColor(hDC,YELLOW);
  double rdist=data->hook->getRaceDistance(),met=data->hook->getMET();
  y=height*8;
  TextO(hDC,width*8,y,height*1  ,"Race Time:   %.2fs",met);
  TextO(hDC,width*8,y,height*1  ,"%s",data->hook->getFileName());
  TextO(hDC,width*8,y,height*1  ,"Total dis:   %.2f km",rdist);
  TextO(hDC,width*8,y,height*0  ,"Avg. Vel :   %.2f km/h",rdist/met*3600);
  return; 
 }
 if (!strcmp(data->hook->getFileName(),"none"))
 {
  TextO(hDC,width*8,y,height*1,"Load a checkpoint");
  return;
 }
 SetTextColor(hDC,GREEN);
 switch(data->hook->getStatus()){
  case IDLE:  TextO(hDC,width,y,height*1  ,"status           :  Idle");break;
  case RACING:TextO(hDC,width,y,height*1  ,"status           :  Racing");break;
 }
 TextO(hDC,width,y,height*1.6,"Elapsed Time     :  %.2fs",data->hook->getMET());

 VESSEL * v=data->hook->vhook();
 double lon=0,lat=0;
 double d1=fabs(data->hook->getDistance());
 SetTextColor(hDC,WHITE);
 TextO(hDC,width,y,height*1  ,"%s",data->hook->getChkName());
 SetTextColor(hDC,GREEN);
 TextO(hDC,width,y,height*1  ,"Radius           :  %.2fm",data->hook->getRadius());
 
 double alt=v->GetAltitude();
 double athresh=data->hook->getThresh();
 double talt=data->hook->getAltitude();
 if(alt>talt+athresh || alt<talt-athresh){
  SetTextColor(hDC,RED);
  if(d1<data->hook->getAltWarnDist()/1000){
   if(fmod(oapiGetSimTime(),1.0)<0.5) TextO(hDC,width,y,height*1  ,"Altitude         :  %.2fkm",data->hook->getAltitude()/1000);
  }else                               TextO(hDC,width,y,height*1  ,"Altitude         :  %.2fkm",data->hook->getAltitude()/1000);
 }else                                TextO(hDC,width,y,height*1  ,"Altitude         :  %.2fkm",data->hook->getAltitude()/1000);
  
 SetTextColor(hDC,GREEN);
 y+=height*1;
 TextO(hDC,width,y,height*1  ,"Alt. Threshold   :  %.2fm",data->hook->getThresh());
 double maxspeed=data->hook->getMaxSpeed();
 if(maxspeed==0){
  TextO(hDC,width,y,height*1  ,"Max Pass Vel.    :  None");
 }else{
  double vel=v->GetAirspeed();
  if (vel>maxspeed){
   SetTextColor(hDC,RED);
   if(d1<data->hook->getVelWarnDist()/1000){
    if(fmod(oapiGetSimTime(),1.0)<0.5)TextO(hDC,width,y,height*1  ,"Max Pass Vel.    :  %.2fm/s",maxspeed);
   }else                              TextO(hDC,width,y,height*1  ,"Max Pass Vel.    :  %.2fm/s",maxspeed);
  }else                               TextO(hDC,width,y,height*1  ,"Max Pass Vel.    :  %.2fm/s",maxspeed);
 }
 SetTextColor(hDC,GREEN);
 TextO(hDC,width,y,height*1  ,"Distance         :  %.2fkm",d1);
 SetTextColor(hDC,WHITE);
 TextO(hDC,width,y,height*1  ,"Bearing          :  %.2f (%.2f)",data->hook->getRelBearing(),data->hook->getBearing());
 SetTextColor(hDC,GREEN);

 y+=height*1;
 SetTextColor(hDC,WHITE);
 TextO(hDC,width,y,height*1  ,"%s",data->hook->getFileName());
 SetTextColor(hDC,GREEN);
 TextO(hDC,width,y,height*1  ,"Checkpoint Count :  %d",data->hook->getCheckpointCount());
 TextO(hDC,width,y,height*1  ,"Total Distance   :  %.2f km",data->hook->getRaceDistance());
 data->hook->getChkCoords(lon,lat,data->iSelectedCheckPoint);
 TextO(hDC,width,y,height*1  ,"%s %.5f %.5f",data->hook->getChkName(data->iSelectedCheckPoint),lon*DEG,lat*DEG);
 TextO(hDC,width,y,height*1  ,"Radius           :  %.2fm",data->hook->getRadius(data->iSelectedCheckPoint));
 TextO(hDC,width,y,height*1  ,"Altitude         :  %.2fkm",data->hook->getAltitude(data->iSelectedCheckPoint)/1000);
 TextO(hDC,width,y,height*1  ,"Alt. Threshold   :  %.2fm",data->hook->getThresh(data->iSelectedCheckPoint));  
 TextO(hDC,width,y,height*1  ,"Distance         :  %.2fkm",data->hook->getDistance(data->iSelectedCheckPoint));
}    
//############################################################################//
//############################################################################//
void RCPMFD::DrawPageVesselInfo(HDC hDC)
{
 Title(hDC,"Contestants Information Page");
 int y=height*2;
 OBJHANDLE hTarget=oapiGetVesselByIndex(data->iSelectedTarget);
 
 if (!oapiIsVessel(hTarget)){data->nextTarget();return;}
 
 VESSEL *v =oapiGetVesselInterface(hTarget);
 VESSEL *me=data->hook->vhook();

 double talt=v->GetAltitude()/1000,alt=me->GetAltitude()/1000;
 double tspd=v->GetAirspeed(),spd=me->GetAirspeed();
 double dalt=alt-talt;
 double dspd=spd-tspd;
 double thead,head,dhead;oapiGetHeading(hTarget,&thead);oapiGetHeading(me->GetHandle(),&head);
 head*=DEG;thead*=DEG;dhead=head-thead;
 VECTOR3 rpos,TF,TW,TA,F,W,A,DA;me->GetRelativePos(hTarget,rpos);
 double distance=length(rpos);
 v->GetForceVector(TF);
 v->GetWeightVector(TW);
 TA=(TF-TW)/v->GetMass();
 me->GetForceVector(F);
 me->GetWeightVector(W);
 A=(F-W)/me->GetMass();
 DA=TA-A;
 
 SetTextColor(hDC,WHITE); 
 TextO(hDC,width,y,height*1  ,"Target    : %s",v->GetName()); 
 SetTextColor(hDC,GREEN);
 TextO(hDC,width,y,height*1  ,"Altitude  : %.2f km",talt);  
 TextO(hDC,width,y,height*1  ,"Airspeed  : %.2f m/s",tspd); 
 TextO(hDC,width,y,height*1  ,"Heading   : %.2f ",thead);
 TextO(hDC,width,y,height*1  ,"Accel.    : %.2f m/s (%.2f Gs)",TA.z,TA.y/G);
 TextO(hDC,width,y,height*1.5,"v.Accel.  : %.2f m/s",TA.y);

 SetTextColor(hDC,WHITE); 
 TextO(hDC,width,y,height*1  ,"Focus     : %s",me->GetName());
 SetTextColor(hDC,GREEN);
 TextO(hDC,width,y,height*1  ,"Altitude  : %.2f km",alt);
 TextO(hDC,width,y,height*1  ,"Airspeed  : %.2f m/s",spd);
 TextO(hDC,width,y,height*1  ,"Heading   : %.2f ",head);
 TextO(hDC,width,y,height*1  ,"Accel.    : %.2f m/s (%.2f Gs)",A.z,A.y/G);
 TextO(hDC,width,y,height*1.5,"v.Accel.  : %.2f m/s",A.y);

 TextO(hDC,width,y,height*1  ,"Rel. Alt. : %.2f km",dalt);
 TextO(hDC,width,y,height*1  ,"Rel. Vel. : %.2f m/s",dspd);
 TextO(hDC,width,y,height*1  ,"Rel. Head.: %.2f ",dhead);
 TextO(hDC,width,y,height*1  ,"Rel. Pos. : %.2f %.2f %.2f km",rpos.x/1000,rpos.y/1000,rpos.z/1000);
 TextO(hDC,width,y,height*1  ,"Distance  : %.2f km",distance/1000);
}   
//############################################################################//
//############################################################################//
//Transform assumptions:
//1. The Earth is flat near the checkpoints (size(course)<<size(Earth))
//2. The race is far from poles
#define xformnear \
 x1=(lon-vlon)*zoom;  \
 y1=(lat-vlat)*zoom;  \
 x= cos(-hdg)*x1+sin(-hdg)*y1; \
 y=-sin(-hdg)*x1+cos(-hdg)*y1; \
 x=wm/2+x; \
 y=hm/2-y

//VC6/VC8 incompatibility hack
double absd(double a){return a<0?-a:a;}
double maxd(double a,double b){return a>=b?a:b;}
//############################################################################//
void RCPMFD::DrawPageRaceMap(HDC hdc)
{ 
 DWORD i;//VC6
 int x,y=10;
 if(!strcmp(data->hook->getFileName(),"none")){
  TextO(hdc,width*8,y,height*1,"Load a checkpoint");
  return;
 }
 double lat,lon,vlat,vlon,valt,xh,yh,xl,yl,zoom,hdg,x1,y1,alt,spd; 
 HPEN red,green,yellow,blue,whdot,gray,old;
 
 //Preparations
 red=CreatePen(PS_SOLID,1,RED);
 green=CreatePen(PS_SOLID,1,GREEN);
 yellow=CreatePen(PS_SOLID,1,YELLOW);
 blue=CreatePen(PS_SOLID,1,BLUE);
 whdot=CreatePen(PS_DASH,1,WHITE);
 gray=CreatePen(PS_DASH,1,GRAY); 
 old=(HPEN)SelectObject(hdc,whdot); 
 RoundRect(hdc,0,0,wm-1,hm-1,15,15);
 
 //Data
 VESSEL *v=oapiGetFocusInterface();
 oapiGetEquPos(v->GetHandle(),&vlon,&vlat,&valt);
 oapiGetHeading(v->GetHandle(),&hdg);
 alt=v->GetAltitude()/1000;
 spd=v->GetAirspeed();    
 double athresh=data->hook->getThresh();
 double talt=data->hook->getAltitude();
 double tspd=data->hook->getMaxSpeed();
 
 //Zoom box,assumptions above
 xh=1e100;yh=1e100;xl=-1e100;yl=-1e100;  
 for(i=0;i<data->hook->getCheckpointCount();i++){ 
  data->hook->getChkCoords(lon,lat,i);
  if(lon-vlon<xh)xh=lon-vlon;
  if(lat-vlat<yh)yh=lat-vlat;
  if(lon-vlon>xl)xl=lon-vlon;
  if(lat-vlat>yl)yl=lat-vlat;
 }
 //Divide by zero prevention
 double mx=maxd(absd(xl-xh),absd(yl-yh));
 if(mx!=0)zoom=(min(wm,hm)/mx)*data->zoom; 
 
 //Text           
 SetTextColor(hdc,GREEN); 
 TextO(hdc,10,y,height,"Zoom      : %.2fx",data->zoom);  
 SetTextColor(hdc,GREEN);
 if(abs(alt*1000-talt)>athresh)SetTextColor(hdc,RED); 
 TextO(hdc,10,y,height,"Altitude  : %.2f/%.2f (~%.2f) Km",alt,talt/1000,athresh/1000);   
 SetTextColor(hdc,GREEN);
 if(spd>tspd)SetTextColor(hdc,RED); 
 TextO(hdc,10,y,height,"Speed     : %.1f/%.1f m/s",spd,tspd);  
       
 //First one                                                      
 data->hook->getChkCoords(lon,lat,0);
 xformnear;
 MoveToEx(hdc,x,y,NULL);
 
 //Checkpoints
 for(i=0;i<data->hook->getCheckpointCount();i++){ 
  data->hook->getChkCoords(lon,lat,i);
  xformnear;
         
  if(i==data->hook->currentCHKPT){
   if( (abs(alt*1000-talt)<athresh)&&(spd<tspd) )SelectObject(hdc,green);else SelectObject(hdc,red);
  }else SelectObject(hdc,blue); 
  Ellipse(hdc,x-5,y-5,x+5,y+5);    
  SelectObject(hdc,gray);  
  LineTo(hdc,x,y);
 }      
 
 //Player
 SelectObject(hdc,yellow); 
 MoveToEx(hdc,wm/2,hm/2,NULL);    
 LineTo(hdc,wm/2,hm/2-20);   
 LineTo(hdc,wm/2-3,hm/2-17);
 LineTo(hdc,wm/2+3,hm/2-17);
 LineTo(hdc,wm/2,hm/2-20);
 
 MoveToEx(hdc,wm/2-5,hm/2,NULL);    
 LineTo(hdc,wm/2+5,hm/2);   
                   
 //Cleanup                   
 SelectObject(hdc,old); 
 DeleteObject(red); 
 DeleteObject(green); 
 DeleteObject(yellow); 
}       
//############################################################################//
//############################################################################//
void RCPMFD::Update(HDC hDC)
{
 switch(data->page){
  case PAGE_MAIN:DrawPageMain(hDC);break;
  case PAGE_VESSELINFO:DrawPageVesselInfo(hDC);break;
  case PAGE_RACEMAP:DrawPageRaceMap(hDC);break;
 }
}
//############################################################################//
