#include <orbitersdk.h>
#include "VOBJ.h"
#include <math.h>
#include "parser.h"

void GetTimeStamp(char * buffer)
{
	SYSTEMTIME time;
	GetLocalTime(&time);
	sprintf(buffer, "%d-%d-%d %d-%d-%d", time.wMonth, time.wDay, time.wYear, time.wHour, time.wMinute, time.wSecond);
}

void VOBJ::TimeStep()
{
	double simt = oapiGetSimTime();
	switch(status)
	{
	case RACING:
		{
			if (timer_started==TRUE)
				met = simt-mst;
			if (lastlogtime<simt)
			{
				VESSEL * v = vhook();
				double lon = 0, lat = 0;
				getcoords(lon, lat);
				lon*=DEG; lat*= DEG;
				double mass = v->GetMass();
				double alt  = v->GetAltitude()/1000;
				double vel  = v->GetAirspeed();
				log("<p><b>Simulation Time: %fs</b><br/>", simt);
				log("Mass     : %f kg<br/>", mass);
				log("Altitude : %f km<br/>", alt);
				log("Velocity : %f m/s<br/>", vel);
				log("Position : %f %f<br/>", lon, lat);
				log("MET      : %f s</p>", met);
				lastlogtime=simt+LOG_TIME_INTERVAL;
			}
			if (isUnderLimits()==TRUE)
			{
				if (currentCHKPT+1 >= checkpoints.size())
				{
					status = IDLE;
					race_complete = TRUE;
					double rdist = getRaceDistance();
					log("<p><b><h1><center>Race Complete</center></h1></b></p>");
					log("Total Distance:  %f km <br/>", getRaceDistance());
					log("Average vel   :  %f km/h <br/>", rdist/met*3600);
					log("Duration      :  %f s <br/>", met);
				}
				else
				{
					if (timer_started==FALSE)
					{
						mst = simt;
						timer_started=TRUE;
						log("<p><b><h2><center>Timer started at %f seconds</center></h2></b></p>", simt);
					}
					log("<p><b><h2><center>%s passed at %f seconds</center></h2></b></p>", getChkName(), simt);
					currentCHKPT += 1;
				}
			}
			break;
		}
	case IDLE:
		{
			if (race_complete==TRUE || checkpoints.size()==0)
				break;
			if (isUnderLimits(0))
			{
				log("<p><b><h2><center>Race timer automatically started at %f with a 10 second penalty</center></h2></b></p>", simt);
				toggleStatus();
				timer_started=TRUE; // override the timer 
				mst = simt-10; // a ten seconds penalty
			}
			break;
		}
	}
}
BOOL VOBJ::isUnderLimits(UINT inx)
{
	if (inx >= checkpoints.size())
		return FALSE;
	VESSEL * v = oapiGetVesselInterface(hook);
	double dist = getDistance(inx)*1000; // distance to target checkpoint
	double alt = v->GetAltitude();
	double vel = v->GetAirspeed();
	RCP_CHKPT chk = checkpoints[inx];
	if (dist <= chk.radius && (alt > chk.alt-chk.athresh && alt < chk.alt+chk.athresh) && 
		(chk.maxSpeed==0 || chk.maxSpeed >= vel)) // if maxSpeed = 0, assume no velocity limit
	{
		return TRUE;
	}
	return FALSE;
}
VFSTATUS VOBJ::toggleStatus()
{
	switch(status)
	{
	case IDLE:
		{
			if (checkpoints.size() < 1)
				break;
			double rad = 0, simt = oapiGetSimTime();
			status = RACING; 
			VESSEL(hook).GetEquPos(startLon, startLat, rad);
			if (racelog) fclose(racelog);
			char fname[255], timestamp[255]; 
			GetTimeStamp(timestamp);
			getFileNameshort(chkfname);
			sprintf(fname, "checkpoints/logs/%s_%s.html", chkfname, timestamp);
			racelog = fopen(fname, "w");
			log("<center><b><h1>%s</h1></b></center>", chkfname);
			log("<center><b>%s</b></center>", timestamp);
			log("<p><h1><center><b>Race started: %f with %s (%s) </b></center></h1></p>", 
				simt, vhook()->GetName(), (vhook()->GetClassName() == NULL ? " " : vhook()->GetClassName()));
			break;
		}
	case RACING:
		reset();
		break;
	}
	return status;
}
void VOBJ::reset()
{
	status = IDLE;
	met = mst = currentCHKPT = lastlogtime = 0;
	race_complete = timer_started = FALSE;
	if (racelog)fclose(racelog); racelog=NULL;
}
double VOBJ::getBearing()
{
	double lat1 = 0, lon1 = 0, lat2 = 0, lon2 = 0, rad = 0;
	getChkCoords(lon2, lat2);
	VESSEL(hook).GetEquPos(lon1, lat1, rad);
	return getBearing(currentCHKPT);
}
double VOBJ::getBearing(UINT inx)
{
	if (inx >= checkpoints.size())
		return 0;
	double lat1 = 0, lon1 = 0, lat2 = 0, lon2 = 0, rad = 0;
	getChkCoords(lon2, lat2, inx);
	VESSEL(hook).GetEquPos(lon1, lat1, rad);
	double y = sin(lon2-lon1) * cos(lat2);
	double x = cos(lat1)*sin(lat2) -
		sin(lat1)*cos(lat2)*cos(lon2-lon1);
	double brng = atan2(y, x)*DEG;
	return (int)(brng+360)%360;
}
double VOBJ::getRelBearing()
{
	return getRelBearing(currentCHKPT);
}
double VOBJ::getRelBearing(UINT inx)
{
	if (inx >= checkpoints.size())
		return 0;
	double bearing = getBearing(inx);
	double heading = 0; oapiGetHeading(hook, &heading); heading *= DEG;
	return heading - bearing;
}
double VOBJ::getAltitude(UINT inx)
{
	if (inx >= checkpoints.size())
		return 0;
	return checkpoints[inx].alt;
}
double VOBJ::getRadius(UINT inx)
{
	if (inx >= checkpoints.size())
		return 0;
	return checkpoints[inx].radius;
}
double VOBJ::getThresh(UINT inx)
{
	if (inx >= checkpoints.size())
		return 0;
	return checkpoints[inx].athresh;
}
double VOBJ::getMaxSpeed(UINT inx)
{
	if (inx >= checkpoints.size())
		return 0;
	return checkpoints[inx].maxSpeed;
}
double VOBJ::getDistance()
{
	VESSEL * v = oapiGetVesselInterface(hook);
	OBJHANDLE ref = v->GetSurfaceRef();

	double lat1 = 0, lon1 = 0, lat2 = 0, lon2 = 0, rad = 0;
	getChkCoords(lon2, lat2);
	v->GetEquPos(lon1, lat1, rad);

	double R = oapiGetSize(ref)/1000;
	return getDistance(lon1, lat1, lon2, lat2, R);
}
double VOBJ::getDistance(double lon1, double lat1, double lon2, double lat2, double radius)
{
	return acos(sin(lat1)*sin(lat2) + cos(lat1)*cos(lat2) * cos(lon2-lon1)) * radius;
}
double VOBJ::getRaceDistance()
{
	double lon1 = 1, lat1 = 0, lon2 = 0, lat2 = 0;
	UINT count = checkpoints.size();
	if (count<1)
		return 0;
	if (count<2)
	{
		getcoords(lon1, lat1);
		getChkCoords(lon2, lat2, 0);
		return getDistance(lon1, lat1, lon2, lat2, oapiGetSize(VESSEL(hook).GetSurfaceRef())/1000);
	}
	double dist = 0;
	OBJHANDLE hRef = VESSEL(hook).GetSurfaceRef();
	double rad = oapiGetSize(hRef)/1000;
	for (UINT i = 0; i < count; i++)
	{
		if (i==count-1)
			break;
		getChkCoords(lon1, lat1, i);
		getChkCoords(lon2, lat2, i+1);
		dist += fabs(getDistance(lon1, lat1, lon2, lat2, rad));
	}
	return dist;
}
double VOBJ::getDistance(UINT inx)
{
	if (inx >= checkpoints.size())
		return 0;
	VESSEL * v = oapiGetVesselInterface(hook);
	OBJHANDLE hRef = v->GetSurfaceRef();
	double lat1 = 0, lon1 = 0, lat2 = 0, lon2 = 0;
	getChkCoords(lon1, lat1, inx);
	getcoords(lon2, lat2);
	return getDistance(lon1, lat1, lon2, lat2, oapiGetSize(hRef)/1000);
}
double VOBJ::getChkDistance()
{
	if (checkpoints.size()<1)
		return 0;
	if (currentCHKPT == 0)
	{
		OBJHANDLE hRef = VESSEL(hook).GetSurfaceRef();
		double lon = checkpoints[currentCHKPT].lon*RAD, lat = checkpoints[currentCHKPT].lat*RAD;
		return getDistance(lon, lat, startLon, startLat, oapiGetSize(hRef)/1000);
	}
	OBJHANDLE hRef = VESSEL(hook).GetSurfaceRef();
	RCP_CHKPT chk1 = checkpoints[currentCHKPT], chk2 = checkpoints[currentCHKPT-1];
	return getDistance(chk1.lon*RAD, chk1.lat*RAD, chk2.lon*RAD, chk2.lat*RAD, oapiGetSize(hRef)/1000);
}
double VOBJ::getChkCoords(double &lon, double &lat)
{
	return getChkCoords(lon, lat, currentCHKPT);
}
double VOBJ::getChkCoords(double &lon, double &lat, UINT inx)
{
	if (inx >= checkpoints.size())
	{
		lon = inx; lat = lon;
		return 0;
	}
	lon = checkpoints[inx].lon*RAD;
	lat = checkpoints[inx].lat*RAD;
	return 0;
}
double VOBJ::getcoords(double &lon, double &lat)
{
	double rad = 0;
	VESSEL(hook).GetEquPos(lon, lat, rad);
	return rad;
}
char* VOBJ::getChkName(UINT inx)
{
	if (inx >= checkpoints.size())
		return NULL;
	return checkpoints[inx].name;
}
VECTOR3 LonLatToVector(double lon, double lat, double rad)
{
	VECTOR3 t;
	t.x = rad * sin(lat) * cos(lon);
	t.y = rad * sin(lat) * sin(lon);
	t.z = rad * cos(lat);
	return t;
}
void VOBJ::LoadCheckpoints()
{
	reset();
	// only valid file names are accepted
	/*FILE * fP = NULL;
	if ((fP=fopen(checkpointFile,"r"))==0)
	{
	checkpoints.clear();
	return;
	}
	else
	fclose(fP);
	*/
	PARSER gParser(checkpointFile);
	gParser.read();
	char cbuf[255];
	DWORD i;

	DWORD chkpCount = gParser.Int("CheckpointCount");
	if (chkpCount<=0)
		return;
	checkpoints.clear();
	RCP_CHKPT checkpoint;
	for (i = 0; i < chkpCount; i++)
	{
		sprintf(cbuf, "Checkpoint_%d_Lon", i+1);
		checkpoint.lon = gParser.Double(cbuf);
		sprintf(cbuf, "Checkpoint_%d_Lat", i+1);
		checkpoint.lat = gParser.Double(cbuf);
		sprintf(cbuf, "Checkpoint_%d_Alt", i+1);
		checkpoint.alt = gParser.Double(cbuf);
		sprintf(cbuf, "Checkpoint_%d_AltThresh", i+1);
		checkpoint.athresh = gParser.Double(cbuf);
		sprintf(cbuf, "Checkpoint_%d_Radius", i+1);
		checkpoint.radius = gParser.Double(cbuf);
		sprintf(cbuf, "Checkpoint_%d_MaxSpeed", i+1);
		checkpoint.maxSpeed = gParser.Int(cbuf);
		sprintf(cbuf, "Checkpoint_%d_AltWarnDist", i+1);
		checkpoint.alt_wardist = gParser.Int(cbuf);
		sprintf(cbuf, "Checkpoint_%d_VelWarnDist", i+1);
		checkpoint.vel_wardist = gParser.Int(cbuf);
		sprintf(cbuf, "Checkpoint_%d_Name", i+1);
		gParser.String(cbuf, checkpoint.name);
		sprintf(cbuf, "Checkpoint_%d_Heading", i+1);
		checkpoint.heading = gParser.Double(cbuf);
		checkpoints.push_back(checkpoint);
	}
	char classname[255];
	OBJHANDLE hv = NULL; VESSEL * v = NULL;
	for (i = 0; i < oapiGetVesselCount(); i++)
	{
		hv = oapiGetVesselByIndex(i);
		if (!oapiIsVessel(hv))
			continue;
		v = oapiGetVesselInterface(hv);
		if (v->GetClassName()==NULL)
			continue;
		strcpy(classname, v->GetClassName());
		if (!strcasecmp(classname, "ORRL_CHKPT") || !strcasecmp(classname, "ORRL_Finish") || !strcasecmp(classname, "ORRL_Start"))
		{
			oapiDeleteVessel(hv);
		}
	}
	VESSELSTATUS vs; vhook()->GetStatus(vs);
	vs.status=1;
	DWORD chkcount = checkpoints.size();
	char name[255];
	for (i = 0; i < chkcount; i++)
	{
		vs.vdata[0].x=checkpoints[i].lon*RAD;
		vs.vdata[0].y=checkpoints[i].lat*RAD;
		vs.vdata[0].z=checkpoints[i].heading*RAD;
		if (i == 0)
		{
			oapiCreateVessel("ORRL_Start", "ORRL_Start", vs);
			continue;
		}
		if (i == chkcount-1)
		{
			oapiCreateVessel("ORRL_Finish", "ORRL_Finish", vs);
			continue;
		}
		sprintf(name, "ORRL_Checkpoint %d", i);
		oapiCreateVessel(name, "ORRL_CHKPT", vs);
	}

}
void VOBJ::LoadCheckpoints(char *file)
{
	if (strlen(file)==0)
	{
		checkpoints.clear();
		return;
	}
	strcpy(checkpointFile, file);
	LoadCheckpoints();
}