#ifndef _H_COMPUTEREX_RACER_CHECKPOINT_MFD_VOBJ_SUPER_VESSEL_CLASS_H_
#define _H_COMPUTEREX_RACER_CHECKPOINT_MFD_VOBJ_SUPER_VESSEL_CLASS_H_

#include <stdarg.h>
#include <orbitersdk.h>
#include "parser.h"
#include <vector>

#pragma warning(disable:4244)

#define LOG_TIME_INTERVAL 1.0

enum VFSTATUS { IDLE = 0, RACING};

void GetTimeStamp(char * buffer);

class RCP_CHKPT
{
public:
	RCP_CHKPT()
	{
		lon = lat = alt = athresh = radius = maxSpeed = alt_wardist = vel_wardist = heading = 0;
		strcpy(name, "");
	}
	RCP_CHKPT(double lon, double lat, double alt, double athresh, double radius, double maxSpeed, char * name, double heading)
	{
		this->lon = lon; this->lat = lat; this->alt = alt; this->athresh = athresh; this->radius = radius; 
		this->maxSpeed=maxSpeed;
		strcpy(this->name, name);
	}
	double lon, lat, alt, athresh, radius, maxSpeed, alt_wardist, vel_wardist, heading;
	char name[255];
};

class VOBJ
{
public:
	VOBJ()
	{
		hook = NULL;
		status = IDLE;
		strcpy(checkpointFile, "none");
		met = mst = currentCHKPT = lastlogtime = 0;
		race_complete=timer_started=FALSE;
		racelog=NULL;
		//LoadCheckpoints();
	}
	VOBJ (OBJHANDLE hook)
	{
		this->hook=hook;
		status = IDLE;
		strcpy(checkpointFile, "none");
		met = mst = currentCHKPT = lastlogtime = 0;
		race_complete=timer_started=FALSE;
		racelog=NULL;
		//LoadCheckpoints();
	}
	void      TimeStep();
	OBJHANDLE getHook() { return hook; }
	VFSTATUS  getStatus() { return status; }
	VFSTATUS  toggleStatus();
	void      setHook(OBJHANDLE h) { hook = h; }
	VESSEL *  vhook() { return oapiGetVesselInterface(hook); }
	void      reset();

	UINT      getCheckpointCount() { return checkpoints.size(); }
	char*     getFileName() { return checkpointFile; }
	char*     getFileNameshort(char * buff)
	{ 
		DWORD i;
		for (i = strlen(checkpointFile); i > 0; i--)
		{
			if (checkpointFile[i] == '/' || checkpointFile[i] == '\\')
			{
				break;
			}
		}
		char * ptr = checkpointFile+i+1;
		strcpy(buff, ptr);
		return buff;
	}
	char*     getChkName() { return getChkName(currentCHKPT); }
	char*     getChkName(UINT inx);
	double    getDistance();
	double    getDistance(double lon1, double lat1, double lon2, double lat2, double radius);
	double    getDistance(UINT inx);
	double    getChkDistance();
	double    getBearing();
	double    getBearing(UINT inx);
	double    getRelBearing();
	double    getRelBearing(UINT inx);
	double    getAltitude() { return getAltitude(currentCHKPT); }
	double    getAltitude(UINT inx);
	double    getRadius() { return getRadius(currentCHKPT); }
	double    getRadius(UINT inx);
	double    getThresh() { return getThresh(currentCHKPT); }
	double    getThresh(UINT inx);
	double    getMaxSpeed() { return getMaxSpeed(currentCHKPT); }
	double    getMaxSpeed(UINT inx);
	double    getChkCoords(double &lon, double &lat);
	double    getChkCoords(double &lon, double &lat, UINT inx);
	double    getcoords(double &lon, double &lat);
	double    getRaceDistance();
	double    getAltWarnDist() { return checkpoints[currentCHKPT].alt_wardist; }
	double    getVelWarnDist() { return checkpoints[currentCHKPT].vel_wardist; }
	double    getMET() { return met; }
	void      LoadCheckpoints(char * file);
	void      setMST(double mst) { this->mst = mst; }
	void      setTimer(BOOL timer) { this->timer_started = timer; }
	BOOL      getTimer() { return timer_started; }
	BOOL      isRaceFinished() { return race_complete; }
	BOOL      isUnderLimits() { return isUnderLimits(currentCHKPT); }
	BOOL      isUnderLimits(UINT inx);
private:
	void	  LoadCheckpoints();
	void      log(const char * fmt, ...)
	{
		if (!racelog)
			return;
		char cbuf[255];   
		va_list arg;
		va_start(arg,fmt);
		vsprintf(cbuf,fmt,arg);
		va_end(arg);
		fputs(cbuf, racelog);
	}
private:
	OBJHANDLE hook;
	VFSTATUS  status;
	char      checkpointFile[64];
	std::vector<RCP_CHKPT> checkpoints;
	double    met, mst, startLon, startLat, lastlogtime;
	BOOL      race_complete, timer_started;
	FILE *    racelog;
	char      chkfname[255];
public:	
	UINT      currentCHKPT;
};

#endif